﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectMonorail.Entities;

namespace ProjectMonorail.Drawings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawingMonorail : DrawingTrain
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="wheels">Количество колёс</param>
    /// <param name="mainColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="rail">Признак наличия магнитного рельса</param>
    /// <param name="secondCarriage">Признак наличия второго вагона</param>
    public DrawingMonorail(int speed, double weight, int wheels, Color mainColor, Color additionalColor, bool rail, bool secondCarriage) : base(40, 150)
    {
        EntityTrain = new EntityMonorail(speed, (int)weight, wheels, mainColor, additionalColor, rail, secondCarriage);
    }


    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public override void DrawTransport(Graphics g)
    {
        if (EntityTrain == null || EntityTrain is not EntityMonorail monoRail || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        base.DrawTransport(g);

        Brush additionalBrush = new SolidBrush(monoRail.AdditionalColor);
        Pen pen = new(Color.Black);
        Brush brBlue = new SolidBrush(Color.LightBlue);
        Brush brWhite = new SolidBrush(Color.White);
        Brush brBlack = new SolidBrush(Color.Black);

        //магнитная рельса
        if (monoRail.Rail)
        {
            if (monoRail.SecondСarriage)
            {
                g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value + 40, 150, 3);
            }
            else
            {
                g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value + 40, 90, 3);
            }
        }

        //Второй вагон
        if (monoRail.SecondСarriage)
        {
            //кузов
            g.DrawRectangle(pen, _startPosX.Value + 95, _startPosY.Value, 55, 30);
            g.FillRectangle(additionalBrush, _startPosX.Value + 96, _startPosY.Value + 1, 54, 29);
            //сцепка
            g.FillRectangle(brBlack, _startPosX.Value + 85, _startPosY.Value + 5, 10, 23);
            //Окно
            g.DrawRectangle(pen, _startPosX.Value + 105, _startPosY.Value + 5, 35, 10);
            g.FillRectangle(brBlue, _startPosX.Value + 106, _startPosY.Value + 6, 34, 9);
            //Дверь
            g.DrawRectangle(pen, _startPosX.Value + 105, _startPosY.Value + 5, 10, 23);
            g.FillRectangle(brWhite, _startPosX.Value + 106, _startPosY.Value + 16, 9, 12);
            //Колёса
            g.DrawEllipse(pen, _startPosX.Value + 100, _startPosY.Value + 30, 10, 10);
            g.DrawEllipse(pen, _startPosX.Value + 110, _startPosY.Value + 30, 10, 10);
            g.DrawEllipse(pen, _startPosX.Value + 130, _startPosY.Value + 30, 10, 10);
            g.DrawEllipse(pen, _startPosX.Value + 140, _startPosY.Value + 30, 10, 10);
        }

        switch (monoRail.Wheels)
        {
            case 3:
                g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
                g.FillEllipse(brWhite, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
                break;
            case 4:
                g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
                g.FillEllipse(brWhite, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
                g.DrawEllipse(pen, _startPosX.Value + 50, _startPosY.Value + 30, 10, 10);
                g.FillEllipse(brWhite, _startPosX.Value + 50, _startPosY.Value + 30, 10, 10);
                break;
        }


    }
}