﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectMonorail.Drawings;
using ProjectMonorail.MovementStrategy;

namespace ProjectMonorail
{
    /// <summary>
    /// Форма работы с объектом "Монорельс"
    /// </summary>
    public partial class FormMonorail : Form
    {
        /// <summary>
        /// Поле-объект для прорисовки объекта
        /// </summary>
        private DrawingTrain? _drawingTrain;

        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;

        /// <summary>
        /// Конструктор формы
        /// </summary>
        public FormMonorail()
        {
            InitializeComponent();
            _strategy = null;
        }

        /// <summary>
        /// Метод прорисовки
        /// </summary>
        private void Draw()
        {
            if (_drawingTrain == null)
            {
                return;
            }

            Bitmap bmp = new(pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingTrain.DrawTransport(gr);
            pictureBoxMonorail.Image = bmp;
        }

        private void CreateObject(string type)
        {
            Random random = new();
            switch (type)
            {
                case nameof(DrawingTrain):
                    _drawingTrain = new DrawingTrain(random.Next(100, 300), random.Next(1000, 3000),
                        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                    break;
                case nameof(DrawingMonorail):
                    _drawingTrain = new DrawingMonorail(random.Next(100, 300), random.Next(1000, 3000), random.Next(2, 5),
                        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                        Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                        Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                    break;
                default: return;
            }

            _drawingTrain.SetPictureSize(pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            _drawingTrain.SetPosition(random.Next(10, 100), random.Next(10, 100));
            _strategy = null;
            comboBoxStrategy.Enabled = true;
            Draw();
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать монорельс"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CreateButton_Click(object sender, EventArgs e)
        {
            CreateObject(nameof(DrawingMonorail));
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать поезд"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CreateTrainButton_Click(object sender, EventArgs e)
        {
            CreateObject(nameof(DrawingTrain));
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingTrain == null)
            {
                return;
            }

            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawingTrain.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    result = _drawingTrain.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    result = _drawingTrain.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    result = _drawingTrain.MoveTransport(DirectionType.Right);
                    break;
            }
            if (result)
            {
                Draw();
            }
        }

        private void buttonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawingTrain == null)
            {
                return;
            }

            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveableTrain(_drawingTrain), pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }
}
