﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMonorail.MovementStrategy;

public class ObjectParameters
{
    /// <summary>
    /// Координата X
    /// </summary>
    private readonly int _x;

    /// <summary>
    /// Координата Y
    /// </summary>
    private readonly int _y;

    /// <summary>
    /// Высота объекта
    /// </summary>
    private readonly int _height;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    private readonly int _width;

    /// <summary>
    /// Левая граница
    /// </summary>
    public int LeftBorder => _x;

    /// <summary>
    /// Верхняя граница
    /// </summary>
    public int TobBorder => _y;

    /// <summary>
    /// Правая граница
    /// </summary>
    public int RightBorder => _x + _width;

    /// <summary>
    /// Нижняя граница
    /// </summary>
    public int DownBorder => _y + _height;

    /// <summary>
    /// Середина объекта по горизонтали
    /// </summary>
    public int ObjectMiddleHorizontal => _x + _width / 2;

    /// <summary>
    /// Середина объекта по вертикали
    /// </summary>
    public int ObjectMiddleVertical => _y + _height / 2;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    /// <param name="height">Высота</param>
    /// <param name="width">Ширина</param>
    public ObjectParameters(int x, int y, int width, int height)
    {
        _x = x;
        _y = y;
        _height = height;
        _width = width;
    }
}
